#import <Foundation/Foundation.h>

#import <sys/socket.h>	// for socket(), AF_INET
#import <netinet/in.h>	// for sockaddr_in
#import <errno.h>	// for errno
#import <arpa/inet.h>	// for inet_aton()
#import <string.h>	// for strerror()

int connectSocket (NSString *address)
{
    // open a socket
    struct sockaddr_in serverAddress;

    serverAddress.sin_len = sizeof (struct sockaddr_in);
    serverAddress.sin_family = AF_INET;

    // make sure this is in network byte order
    serverAddress.sin_port = htons(80); // HTTP port

    // convert from dotted-quad format to a 4-byte integer
    // assumes IPv4 addresses
    if (!inet_aton ([address cString], &serverAddress.sin_addr)) {
	NSLog (@"could not convert '%@' to integer address");
	return (-1);
    }

    // and make sure this is in network byte order too
    serverAddress.sin_addr = htonl (serverAddress.sin_addr);

    // zero out the padding of the serverAddress structure
    memset (&(serverAddress.sin_zero), 0, sizeof(serverAddress.sin_zero));

    // make our socket
    int fd;
    fd = socket (AF_INET, SOCK_STREAM, 0);

    if (fd == -1) {
	NSLog (@"could not make a socket.  Error was %d/%s",
	       errno, strerror(errno));
	return (-1);
    }

    // connect to the other side
    int result;
    result = connect (fd, (struct sockaddr *) &serverAddress,
		      sizeof(serverAddress));

    if (result == -1) {
	NSLog (@"could not connect.  Error was %d/%s",
	       errno, strerror(errno));
	return (-1);
    }

    return (fd);

} // connectSocket



int main (int argc, const char *argv[]) 
{
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

    // find the host address
    NSHost *host;
    host = [NSHost hostWithName: @"borkware.com"];
    
    int fd;
    fd = connectSocket ([host address]);

    if (fd == -1) {
	// couldn't connect, so just exit
	return (-1);
    }

    // wrap an NSFileHandle around the file descriptor
    NSFileHandle *file;
    file = [[NSFileHandle alloc] initWithFileDescriptor: fd];
    

    // this is HTTP, so blort the request
    [file writeData:
	      [@"GET / HTTP/1.0\n" dataUsingEncoding: NSASCIIStringEncoding]];
    [file writeData:
	      [@"Host: borkware.com\n" dataUsingEncoding: NSASCIIStringEncoding]];
    [file writeData:
	      [@"\n" dataUsingEncoding: NSASCIIStringEncoding]];

    // now read the response
    NSData *data;
    data = [file readDataToEndOfFile];

    NSString *string;
    string = [[NSString alloc] initWithData: data
			       encoding: NSASCIIStringEncoding];
    NSLog (@"got %@", string);

    
    [string release];
    [file release]; // closes socket for us.

    [pool release];

    return (0);

} // main
